# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/Handle_data_feed.py $
# Last committed:     $Revision: 424007 $; $Author: svcdms $; $Date: 2019-11-19 11:18:03 +0100 (di, 19 nov 2019) $
#
from DataFeedStatus import DataFeedStatus
from datetime import datetime, timedelta
from dmsDb.DmsDb import DmsDb
from dmsLogger.DmsLogger import DmsLogger
from dmsSvn.DmsSvn import DmsSvn
from filecmp import cmp
from ftpImport.DmsFtp import DmsFtp
from misc.Utils import Utils
from os import path
from re import sub
from svn import local, exception
import getopt
import sys


class Handle_data_feed:
    @staticmethod
    def commit_status_files(folder, fn):
        lc = local.LocalClient(folder)
        lc.cleanup()
        lc.update()
        try:
            lc.add(fn)
        except exception.SvnException:
            pass
        lc.commit("commit from automatic process")

    def __init__(self, folder_settings, sensor):
        self._folder_settings = folder_settings
        self._logger = DmsLogger.set_logger(self._folder_settings["log_folder"], r"dms-ftp-svn.out", "ftp-log-" + sensor)
        self._logger.info("Handle_data_feed.init: $Revision: 424007 $")
        self._db = DmsDb(self._logger)
        self._sensor = sensor

    def ftp_magic(self, ftp_dms, remote_folder, file, svn):

        # download the file twice
        primary_file = (self._folder_settings["primary_root"] + remote_folder + "/" + file)
        secondary_file = (self._folder_settings["secondary_ftp"] + remote_folder + "/" + file)

        self._logger.debug("calling ftp.do_bin_ftp({}, {}, {})".format(remote_folder, file, primary_file))
        self._logger.debug("calling ftp.do_bin_ftp({}, {}, {})".format(remote_folder, file, secondary_file))
        if ftp_dms.do_bin_ftp(remote_folder, file, primary_file) & ftp_dms.do_bin_ftp(remote_folder, file, secondary_file):
            # compare the files
            if cmp(primary_file, secondary_file, shallow="FALSE"):
                self._logger.debug("Handle_data_feed.ftp_magic: the files with name {} are identical".format(file))
                svn.add_file(remote_folder, file)
                svn.commit()
                #SVN update uitgezet omdat tot op heden geen fouten geconstateerd zijn met het inchecken van bestanden in SVN (ruimtebesparing)
				#if svn.no_diff(remote_folder, file):
                    # remove(secondary_file)
                    # The secondary files will be picked up by the raw data import process
                #    return True
                #else:
                #    return False
                return True
            else:
                self._db.add_file_to_list(remote_folder,self._folder_settings["primary_root"],file,"ftp_compare",datetime.now(),datetime.now(),1,)
                self._db.add_file_to_list(remote_folder,self._folder_settings["secondary_ftp"],file,"ftp_compare",datetime.now(),datetime.now(),1,)
                self._logger.warning("Handle_data_feed.ftp_magic: we have a problem with ftp {}".format(file))
                return False

    #
    # main process
    def run(self):
        # open ftp connection
        try:
            ftp_dms = DmsFtp(self._logger, self._db)
            tree = []
            tree = ftp_dms.recurse_ftp_tree("/" + self._sensor, tree)
            self._logger.debug("Tree {}".format(tree))

            for remote_folder in tree:
                try:
                    self._logger.info("Downloading remote folder {}".format(remote_folder))

                    fn = sub("/", "_", remote_folder).upper() + "_status.json"
                    status_file = self._folder_settings["status_folder"] + "/" + fn

                    # retrieve the latest status of the data feed
                    data_feed_status = DataFeedStatus(status_file, self._logger)
                    most_recent_file_read = data_feed_status.get_timestamp_from_file()
                    old_timestamp = most_recent_file_read["ts"]
                    self._logger.info("Handle_data_feed.run: all files more recent than {} will be transferred".format(old_timestamp))
                except FileNotFoundError:
                    raise
                else:
                    # get list of
                    file_timestamps = ftp_dms.get_timestamp_of_files(remote_folder)
                    if len(file_timestamps) == 0:
                        continue

                    # initiate the svn module
                    self._logger.debug("Initiating svn {}".format(self._sensor))
                    svn = DmsSvn(self._folder_settings, self._logger, self._db, remote_folder)

                    # determine the moment x minutes ago
                    not_too_new = datetime.now() - timedelta(minutes=3)

                    # loop through the list order of the timestamps
                    for fn, ts in sorted(file_timestamps.items(), key=lambda item: (item[1], item[0])):
                        # skip file that are older than the timestamp
                        if old_timestamp < ts < not_too_new:
                            self._logger.debug("Handle_data_feed.run: transferring %s: %s" % (fn, ts))
                            if self.ftp_magic(ftp_dms, remote_folder, fn, svn):
                                data_feed_status.update_timestamp_file(fn, ts)

                    # commit status files
                    if path.isfile(status_file):
                        self.commit_status_files(self._folder_settings["status_folder"], status_file)
        finally:
            ftp_dms.close_connection()
            self._db.close()
            self._logger.info("Handle_data_feed.run: ready for now")


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["sensor="])
    except getopt.GetoptError:
        print("Handle_data_feed.py -s <sensor>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("Handle_data_feed.py -s <sensor>")
            sys.exit()
        elif opt in ("-s", "--sensor"):
            sensor = arg
            folder_settings = Utils.read_folder_settings()
            try:
                df_handle = Handle_data_feed(folder_settings, sensor)
                df_handle.run()
            except FileNotFoundError:
                exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
